-- --------------------
-- TellMeWhen
-- Originally by Nephthys of Hyjal <lieandswell@yahoo.com>

-- Other contributions by:
--		Sweetmms of Blackrock, Oozebull of Twisting Nether, Oodyboo of Mug'thol,
--		Banjankri of Blackrock, Predeter of Proudmoore, Xenyr of Aszune

-- Currently maintained by
-- Cybeloras of Mal'Ganis
-- --------------------


if not TMW then return end

local TMW = TMW
local L = TMW.L
local print = TMW.print

local format = format
local isNumber = TMW.isNumber

local Processor = TMW.Classes.IconDataProcessor:New("DURATION", "start, duration")
Processor.SIUVs[#Processor.SIUVs+1] = "local OnGCD = TMW.OnGCD"

function Processor:CompileFunctionSegment(t)
	-- GLOBALS: start, duration
	t[#t+1] = [[
	duration = duration or 0
	start = start or 0
	
	if duration == 0.001 then duration = 0 end -- hardcode fix for tricks of the trade. nice hardcoding on your part too, blizzard
	local d = duration - (TMW.time - start)
	d = d > 0 and d or 0

	if EventHandlersSet.OnDuration then
		if d ~= icon.__lastDur then
			icon:QueueEvent("OnDuration")
			icon.__lastDur = d
		end
	end

	if attributes.start ~= start or attributes.duration ~= duration then

		local realDuration = OnGCD(duration) and 0 or duration -- the duration of the cooldown, ignoring the GCD
		if icon.__realDuration ~= realDuration then
			-- detect events that occured, and handle them if they did
			if realDuration == 0 then
				if EventHandlersSet.OnFinish then
					icon:QueueEvent("OnFinish")
				end
			else
				if EventHandlersSet.OnStart then
					icon:QueueEvent("OnStart")
				end
			end
			icon.__realDuration = realDuration
		end

		TMW:Fire(DURATION.changedEvent, icon, start, duration, d)
		doFireIconUpdated = true

		attributes.start = start
		attributes.duration = duration
	end
	--]]
end

Processor:AddDogTag("TMW", "Duration", {
	code = function (groupID, iconID)
		local group = TMW[groupID]
		local icon = group and group[iconID]
		if icon then
			local attributes = icon.attributes
			local duration = attributes.duration - (TMW.time - attributes.start)
			if duration <= 0 then
				return 0
			end

			-- cached version of tonumber()
			return isNumber[format("%.1f", duration)] or 0
		else
			return 0
		end
	end,
	arg = {
		'group', 'number', '@req',
		'icon', 'number', '@req',
	},
	events = "FastUpdate",
	ret = "number",
	doc = L["DT_DOC_Duration"],
	example = '[Duration] => "5.462"; [Duration:TMWFormatDuration] => "5.4"; [Duration(4, 5)] => "97.32156"; [Duration(4, 5):TMWFormatDuration] => "1:37"',
	category = L["ICON"],
})

TMW:RegisterCallback("TMW_ICON_SETUP_PRE", function(event, icon)
	if not TMW.Locked then
		icon:SetInfo("start, duration", 0, 0)
	end
end)
